﻿using ContextSensingClient;
using SampleApp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WALWOASampleApp.CallBackFunctions
{
	class FeatureCallback : IFeatureCallback
	{
		Logger logger = new Logger();

		public void OnError(FeatureType featureType, Error state)
		{
			try
			{
				logger.Error("Received error " + state.Description + " from service");
				switch (state.ErrorType)
				{
					case ErrorType.ServiceStopped:
					case ErrorType.ServiceUnavailable:
						{
							SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
							{
								SampleAppProgram.walwoaForm.ClearFeatureCheckboxes();
								MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
								SampleAppProgram.walwoaForm.EnableFeatureCheckboxes();
							}));
						}
						break;
					case ErrorType.ServiceRecoveryInProgress:
						{
							SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
							{
								SampleAppProgram.walwoaForm.ClearFeatureCheckboxes();
								SampleAppProgram.walwoaForm.EnableFeatureCheckboxes(false);
								logger.Debug(state.Description);
								MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
							}));
						}
						break;
					case ErrorType.SetOptionError:
						{
							MessageBox.Show(state.Description, "Setting Option for " + featureType + "Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
							logger.Error(state.Description);
						}
						break;
					default:
						{
							switch (featureType)
							{
								case FeatureType.LOCK:
									SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
									{
										SampleAppProgram.walwoaForm.walCheckBox.CheckedChanged -= SampleAppProgram.walwoaForm.checkBoxWAL_CheckedChanged;
										if (SampleAppProgram.walwoaForm.walCheckBox.CheckState == CheckState.Checked)
										{
											SampleAppProgram.walwoaForm.walCheckBox.CheckState = CheckState.Unchecked;
											SampleAppProgram.walwoaForm.UpdateWALControlsVisibility(false);
											logger.Error(state.Description);
										}
										SampleAppProgram.walwoaForm.walCheckBox.CheckedChanged += SampleAppProgram.walwoaForm.checkBoxWAL_CheckedChanged;
										MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
									}));
									break;
								case FeatureType.WAKE:
									SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
									{
										SampleAppProgram.walwoaForm.woaCheckBox.CheckedChanged -= SampleAppProgram.walwoaForm.checkBoxWOA_CheckedChanged;
										if (SampleAppProgram.walwoaForm.woaCheckBox.CheckState == CheckState.Checked)
										{
											SampleAppProgram.walwoaForm.woaCheckBox.CheckState = CheckState.Unchecked;
											SampleAppProgram.walwoaForm.UpdateWOAControlsVisibility(false);
											logger.Error(state.Description);
										}
										SampleAppProgram.walwoaForm.woaCheckBox.CheckedChanged += SampleAppProgram.walwoaForm.checkBoxWOA_CheckedChanged;
										MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
									}));
									break;
								case FeatureType.NOLOCKONPRESENCE:
									SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
									{
										SampleAppProgram.walwoaForm.nlopCheckBox.CheckedChanged -= SampleAppProgram.walwoaForm.checkBoxNlop_CheckedChanged;
										if (SampleAppProgram.walwoaForm.nlopCheckBox.CheckState == CheckState.Checked)
										{
											SampleAppProgram.walwoaForm.nlopCheckBox.CheckState = CheckState.Unchecked;
											SampleAppProgram.walwoaForm.UpdateNLoPControlsVisibility(false);
											logger.Error(state.Description);
										}
										SampleAppProgram.walwoaForm.nlopCheckBox.CheckedChanged += SampleAppProgram.walwoaForm.checkBoxNlop_CheckedChanged;
										MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
									}));
									break;
								case FeatureType.ADAPTIVEDIMMING:
									SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
									{
										SampleAppProgram.walwoaForm.adCheckBox.CheckedChanged -= SampleAppProgram.walwoaForm.checkBoxAd_CheckedChanged;
										if (SampleAppProgram.walwoaForm.adCheckBox.CheckState == CheckState.Checked)
										{
											SampleAppProgram.walwoaForm.adCheckBox.CheckState = CheckState.Unchecked;
											SampleAppProgram.walwoaForm.UpdateAdControlsVisibility(false);
											logger.Error(state.Description);
										}
										SampleAppProgram.walwoaForm.adCheckBox.CheckedChanged += SampleAppProgram.walwoaForm.checkBoxAd_CheckedChanged;
										MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
									}));
									break;
								case FeatureType.APPLICATION:
									SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
									{
										logger.Error(state.Description);
										MessageBox.Show(state.Description, state.ErrorType.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);
									}));
									break;
							}
						}
						break;
				}
			}
			catch (Exception e)
			{
				logger.Debug(e.Message);
				logger.Debug("Exception Thrown in WAL OnError");
			}

		}

		public void OnEvent(FeatureType featureType, EventType eventType, object eventPayload)
		{
			try
			{
				logger.Info("Event " + eventType + " for feature " + featureType + " received from service. Event payload " + eventPayload?.ToString());
				switch (eventType)
				{
					case EventType.EVENT_EXTERNAL_MONITOR:
						{
							new Thread(() =>
							{
								MessageBox.Show("External Monitor Connected", "External Monitor", MessageBoxButtons.OK);
							}).Start();
						}
						break;
					case EventType.EVENT_WAL:
						{
							//logger.Info("OnEvent: WAL in action " + eventType.ToString());
							new Thread(() =>
							{
								MessageBox.Show("WAL Event", "WAL", MessageBoxButtons.OK);
							}).Start();
						}
						break;
					case EventType.EVENT_WOA:
						{
							//logger.Info("OnEven: WOA in action " + eventType.ToString());
							new Thread(() =>
							{
								MessageBox.Show("WOA Event", "WOA", MessageBoxButtons.OK);
							}).Start();
						}
						break;
					case EventType.PROXIMITY:
						{
							float proximityDistanceMm =  (float) eventPayload;
							SampleAppProgram.walwoaForm.proximityDistanceValueLabel.Text = proximityDistanceMm.ToString();
						}
						break;
					case EventType.EVENT_SERVICE_RECOVERED:
						{
							SampleAppProgram.walwoaForm.Invoke(new MethodInvoker(delegate
							{
								SampleAppProgram.walwoaForm.EnableFeatureCheckboxes(true);
								SampleAppProgram.walwoaForm.RefreshFeaturesViaGetOptions();
							}));
						}
						break;
					case EventType.EVENT_DTT_STARTED:
						{
							new Thread(() =>
							{
								MessageBox.Show("Detected DTT process start", "Process Event", MessageBoxButtons.OK);
							}).Start();
						}
						break;
				}
			}
			catch (Exception e)
			{
				logger.Debug(e.Message);
				logger.Error("Exception on handling OnEvent");
			}
		}

		public void OnSuccess(FeatureType featureType, ResponseType responseType)
		{
			try
			{
				logger.Info("OnSuccess with response " + responseType.ToString() + " from service, for feature "+featureType.ToString());
				switch (featureType)
				{
					case FeatureType.LOCK:
						SampleAppProgram.walwoaForm.walCheckBox.Enabled = true;
						break;
					case FeatureType.WAKE:
						SampleAppProgram.walwoaForm.woaCheckBox.Enabled = true;
						break;
					case FeatureType.NOLOCKONPRESENCE:
						SampleAppProgram.walwoaForm.nlopCheckBox.Enabled = true;
						break;
					case FeatureType.ADAPTIVEDIMMING:
						SampleAppProgram.walwoaForm.adCheckBox.Enabled = true;
						break;
					case FeatureType.APPLICATION:
						logger.Info("Enable/Disable success for Privacy " + responseType.ToString());
						String successMsg = "";
						switch(responseType)
						{
							case ResponseType.NO_CHANGE:
								successMsg = "No face detection sensor found on the system. Sensor switch did not occur.";
								break;
							case ResponseType.ENABLE_SUCCESS:
								successMsg = "Enabled privacy";
								break;
							case ResponseType.DISABLE_SUCCESS:
								successMsg = "Disabled privacy";
								break;
						}
						new Thread(() =>
						{
							MessageBox.Show(successMsg, "Privacy Switch", MessageBoxButtons.OK);
						}).Start();
						break;
				}
			}
			catch (Exception e)
			{
				logger.Debug(e.Message);
				logger.Error("Exception on handling OnSuccess");
			}
		}
	}
}
